/////////////////////////////////////////////////////////////////
// mainfrm.cpp : implementation of the CMainFrame class

#include "stdafx.h"
#include "winwks.h"
#include "mainfrm.h"
#include "winwkdoc.h"
#include "winwkvw.h"

#define    SIZE_WWSINFO    sizeof WWSINFO
#define    WWS_MENU_IDX    1
#define    NUM_MRU_WWS     4

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)
BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
    //{{AFX_MSG_MAP(CMainFrame)
    ON_COMMAND(ID_WORKSPACE_LOAD, OnWorkspaceLoad)
    ON_COMMAND(ID_WORKSPACE_SAVE, OnWorkspaceSave)
    ON_COMMAND(ID_WORKSPACE_CLOSE, OnWorkspaceClose)
    ON_WM_INITMENUPOPUP()
    ON_WM_CLOSE()
    ON_WM_CREATE()
    ON_COMMAND(ID_WORKSPACE_AUTOFIT, OnWorkspaceAutofit)
    ON_COMMAND(ID_WORKSPACE_CAPTION, OnWorkspaceCaption)
    ON_UPDATE_COMMAND_UI(ID_WORKSPACE_CAPTION, OnUpdateWWSCaption)
    //}}AFX_MSG_MAP
    ON_COMMAND_EX(ID_WORKSPACE_LOAD+1, OnCommandWWS)
    ON_COMMAND_EX(ID_WORKSPACE_LOAD+2, OnCommandWWS)
    ON_COMMAND_EX(ID_WORKSPACE_LOAD+3, OnCommandWWS)
    ON_COMMAND_EX(ID_WORKSPACE_LOAD+4, OnCommandWWS)
END_MESSAGE_MAP()

CMainFrame::CMainFrame() {}
CMainFrame::~CMainFrame() {}

/////////////////////////////////////////////////////////////////
//  Function: GetDocTemplate()
//  Note:     Helper to Get DocTemplate for access doc chain
//            Also can be the member function of CWinwksApp
static CDocTemplate* GetDocTemplate()
{
    CWinApp* pApp = AfxGetApp();
    CDocTemplate* pTemplate;
    
#ifdef WIN32
    POSITION posDocTempl = pApp->GetFirstDocTemplatePosition();
    pTemplate = pApp->GetNextDocTemplate(posDocTempl);
#else
    pTemplate = (CDocTemplate*)pApp->m_templateList.GetHead();
#endif

    ASSERT(pTemplate != NULL);  
    return pTemplate;
}
 
/////////////////////////////////////////////////////////////////
//  Function: GetDocChildFrame()                            
//  Note:     Helper to Get Document frame      
//            Also can be the member function of CMDIChildWnd
static CMDIChildWnd* GetDocChildFrame(CDocument* pDoc)
{   
    POSITION pos = pDoc->GetFirstViewPosition();
    CView* pView = pDoc->GetNextView(pos);  
    ASSERT( pView!= NULL );

    CMDIChildWnd * pChildFrame 
         = (CMDIChildWnd*)pView->GetParentFrame();
    ASSERT( pChildFrame!= NULL ); 
    return pChildFrame;                                                             
}

/////////////////////////////////////////////////////////////////
//  Function: GetDocChildFrame()                            
//  Note:     Helper to check the szPath exist or not      
static BOOL FileExists(LPCSTR szPath)      
{ 
    CFileStatus fs; 
    return CFile::GetStatus(szPath, fs);
}

/////////////////////////////////////////////////////////////////
//  Function: DiagnSpaceload()   
//  Note:     WWS loading for OnDiagnSpaceload(), OnCommandWWS()
void CMainFrame::WorkSpaceload(CString strPath)
{   
    ASSERT( !strPath.IsEmpty() && FileExists(strPath) ); 
      
    CFile fBin(strPath, CFile::modeRead| CFile::typeBinary); 
    
    WWSINFO WwsInfo;   // read main frame placement first
    if ( SIZE_WWSINFO!=fBin.Read(&WwsInfo, SIZE_WWSINFO)) return; 
    
    if (!SetWindowPlacement(&WwsInfo.wp)) return; // Error Placement
    ShowWindow(WwsInfo.wp.showCmd);    // set main frame first
    
    OnWorkspaceClose();       // close all currrent windows
    CDocTemplate*     pTemplate = GetDocTemplate();

    while (SIZE_WWSINFO==fBin.Read(&WwsInfo, SIZE_WWSINFO) )
    {
        pTemplate->OpenDocumentFile(NULL);  // create new MDIChild
        CMDIChildWnd* pChildWnd = MDIGetActive();
        CDocument* pDoc = pChildWnd->GetActiveDocument(); 
            
    // set three application specific data if applicable
        pDoc->SetTitle(WwsInfo.szDocName); 
        if (WwsInfo.wp.showCmd != SW_SHOWMINIMIZED)  
        {
            SetWindowLong(pChildWnd->GetSafeHwnd(), 
                          GWL_STYLE, WwsInfo.lWndStyle);   
        } 

        if (WwsInfo.szDocPath[0] && FileExists(WwsInfo.szDocPath))
        {    // if file is not found, let Doc path name be NULL
            pDoc->SetPathName(WwsInfo.szDocPath); 
        }
            
        if (!pChildWnd->SetWindowPlacement(&WwsInfo.wp)) return;  
        pChildWnd->ShowWindow(WwsInfo.wp.showCmd);       
    }

    fBin.Close();     
    UpdateAryMruWWS(strPath);
}

/////////////////////////////////////////////////////////////////
//  Function: OnDiagnSpaceload() 
//  Note:     message handle        
void CMainFrame::OnWorkspaceLoad()
{
    CFileDialog FDlg(TRUE, "WWS", "*.WWS", 
                     OFN_FILEMUSTEXIST|OFN_HIDEREADONLY,
                     "WWS Files (*.WWS)|*.WWS||");

    FDlg.m_ofn.lpstrTitle = "Load Window Work Space";
    if (FDlg.DoModal() != IDOK) return; 
    if (FDlg.GetFileExt().CompareNoCase("WWS")) return; 

    WorkSpaceload(FDlg.GetPathName()); 
}

/////////////////////////////////////////////////////////////////
//  Function: OnWorkspaceSave() 
//  Note:     message handle     
void CMainFrame::OnWorkspaceSave()
{
    CFileDialog FDlg(FALSE, "WWS", "WSpace.WWS", 
                     OFN_OVERWRITEPROMPT| OFN_HIDEREADONLY,
                     "WWS Files (*.WWS)|*.WWS||");

    FDlg.m_ofn.lpstrTitle = "Save Window Work Space";
    if (FDlg.DoModal() != IDOK) return;
    if (FDlg.GetFileExt().CompareNoCase("WWS")) return; 
    
    WWSINFO WwsInfo ={(UINT)sizeof WINDOWPLACEMENT};  
    if (!GetWindowPlacement(&WwsInfo.wp)) return; // Error Placement
   
    CFile fBin(FDlg.GetPathName(),
          CFile::modeCreate| CFile::modeWrite| CFile::typeBinary);
    fBin.Write(&WwsInfo, SIZE_WWSINFO);  // save main frame first

//  save child window now
    CDocTemplate* pTemplate = GetDocTemplate();
    POSITION      posDoc    = pTemplate->GetFirstDocPosition();
    CDocument*    pDoc      = NULL; 
    
    while (posDoc)
    {
        pDoc = pTemplate->GetNextDoc(posDoc);
        ASSERT( pDoc!= NULL );
        CMDIChildWnd *pFrm = GetDocChildFrame(pDoc); 
        if (!pFrm->GetWindowPlacement(&WwsInfo.wp)) return; 

    // get three application specific data to save into WWS file
        pFrm->GetWindowText(WwsInfo.szDocName, _MAX_FNAME-1);
        strcpy(WwsInfo.szDocPath, (const char*)pDoc->GetPathName());   
        WwsInfo.lWndStyle = GetWindowLong(pFrm->GetSafeHwnd(), 
                                          GWL_STYLE);
        fBin.Write( &WwsInfo, SIZE_WWSINFO );  
    }

    fBin.Close();     
    UpdateAryMruWWS(FDlg.GetPathName());
}

/////////////////////////////////////////////////////////////////
//  Function: OnWorkspaceClose()                 
//  Note:     message handle, clearup before loading WWS        
void CMainFrame::OnWorkspaceClose()
{   
    GetDocTemplate()->CloseAllDocuments(FALSE);
}
     
/////////////////////////////////////////////////////////////////
//  Function: UpdateAryMruWWS()                            
//  Note:     Helper for maintain 4 most recent used WWS  
void CMainFrame::UpdateAryMruWWS(CString strPath)
{
    int iCount =m_aryMruWWS.GetSize();
    ASSERT(iCount<=NUM_MRU_WWS); 

    for (int i= 0; i<iCount; i++) 
    {
        if (strPath==m_aryMruWWS[i])   // if found, delete it 
        {   
            m_aryMruWWS.RemoveAt(i);
            iCount--;
            break;    
        }
    }                              
            
    if (iCount ==NUM_MRU_WWS) // reach the max, delete the last
        m_aryMruWWS.RemoveAt(iCount-1); 
    m_aryMruWWS.InsertAt(0, strPath); // put the recent at first
}              

/////////////////////////////////////////////////////////////////
//  Function: OnInitMenuPopup                                    
//  Note:     Update 4 WWS dynamic menu items       
//            Load                  (ID_WORKSPACE_LOAD)
//            MF_SEPARATOR     
//            D:\....\xxxxxx.WWS    (ID_WORKSPACE_LOAD +1)
void CMainFrame::OnInitMenuPopup(CMenu* pPopupMenu, UINT nIndex, 
                                 BOOL bSysMenu)
{
    CMDIFrameWnd::OnInitMenuPopup(pPopupMenu, nIndex, bSysMenu);
    
    BOOL bMaximized = FALSE;
    MDIGetActive(&bMaximized); 

    if ((int)nIndex ==(bMaximized ? WWS_MENU_IDX+1 : WWS_MENU_IDX)
        && bSysMenu== FALSE )
    {   
        int  iMenuItemCount =pPopupMenu->GetMenuItemCount();
                  
        for (int i=0; i< iMenuItemCount; i++)  
            if ( UINT(ID_WORKSPACE_LOAD)
                      ==pPopupMenu->GetMenuItemID(i) )  break;
                  
        ASSERT(i<iMenuItemCount); // ID_WORKSPACE_LOAD must be found
                
        for (int j=iMenuItemCount-1; j>i; j--) 
                pPopupMenu->RemoveMenu(j, MF_BYPOSITION);
                      
        if (!m_aryMruWWS.GetSize()) return;
        
        for (i= m_aryMruWWS.GetSize()-1; i>=0; i--) 
            if (!FileExists(m_aryMruWWS[i])) 
                m_aryMruWWS.RemoveAt(i); //if not found, erase item
        
        if (m_aryMruWWS.GetSize())
        {
            pPopupMenu->AppendMenu(MF_SEPARATOR); 
            for (i= 0; i<m_aryMruWWS.GetSize(); i++)       
                pPopupMenu->AppendMenu(MF_STRING, 
                                       UINT(ID_WORKSPACE_LOAD)+1+i,
                                       m_aryMruWWS[i]); 
        }
    }
}

/////////////////////////////////////////////////////////////////
//  Function: OnCommandWWS()                                    
//  Note:     On WWS dynamic menu click  
BOOL CMainFrame::OnCommandWWS(UINT nID)
{
    if ( nID<ID_WORKSPACE_LOAD+1 ||       // uId out of range 
         nID>ID_WORKSPACE_LOAD +NUM_MRU_WWS ) return TRUE; 

    CMenu* menu = GetMenu();
    if (!menu) return TRUE;

    BOOL bMaximized = FALSE;
    MDIGetActive(&bMaximized); 
    int nPos = bMaximized ? WWS_MENU_IDX+1 : WWS_MENU_IDX;

    menu = menu->GetSubMenu(nPos);
    if (!menu) return TRUE; // can't get WorkSpace menu

    char  sz[_MAX_PATH];    
    menu->GetMenuString(nID, sz, _MAX_PATH, MF_BYCOMMAND);

    WorkSpaceload(CString(sz));
    return TRUE;
}

/////////////////////////////////////////////////////////////////
//  Function: ReadProfileWWS, WriteProfileWWS, OnClose, OnCreate
//  Note:     Access WWS path from ini  
void CMainFrame::ReadProfileWWS()
{
    m_aryMruWWS.RemoveAll();
    CWinApp *pApp = AfxGetApp();    
    int iCount = pApp->GetProfileInt("Files", "WWS Count", 0);
    for (int i= 0; i<iCount; i++)
        m_aryMruWWS.Add(
           pApp->GetProfileString("Files", 
                                  CString("WWS")+char(i+49)));
}

void CMainFrame::WriteProfileWWS()
{ 
    int iCount =m_aryMruWWS.GetSize();
    ASSERT(iCount<=NUM_MRU_WWS); 
    CWinApp *pApp = AfxGetApp();    
    pApp->WriteProfileInt("Files", "WWS Count", iCount);

    for (int i= 0; i<iCount; i++) 
        pApp->WriteProfileString("Files", 
                                 CString("WWS")+char(i+49),
                                 m_aryMruWWS[i]);
}

void CMainFrame::OnClose()
{
    WriteProfileWWS();
    CMDIFrameWnd::OnClose();
}


int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
    if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1) return -1;
    ReadProfileWWS();
    return 0;
}


/////////////////////////////////////////////////////////////////
//  Function: OnWorkspaceAutofit()             
//  Note:     message handle, Active child window Autofit      
void CMainFrame::OnWorkspaceAutofit()
{
    CMDIChildWnd* pFrm = MDIGetActive();  
    if (!pFrm) return;    // no Active child window
    
    CDocument* pDoc = pFrm->GetActiveDocument(); 
    POSITION pos = pDoc->GetFirstViewPosition();
    CWksView* pView = (CWksView*)pDoc->GetNextView(pos);  
    ASSERT( pView!= NULL );  

    CRect rcMainCli;
    GetClientRect(rcMainCli);  // get main frame client window
    
// get child frame non client and scroll bar sizes for adgusting
    CRect rc;
    CSize sizeNc, sizeScr;  
    pFrm->GetWindowRect(rc);  
    sizeNc = CSize( rc.Width()+5, rc.Height()+5);
    pFrm->GetClientRect(rc);
    sizeNc -= CSize( rc.right, rc.bottom );  
    
    sizeScr = CSize(GetSystemMetrics(SM_CXHSCROLL), 
                    GetSystemMetrics(SM_CXVSCROLL));
    
    WINDOWPLACEMENT wp = {sizeof WINDOWPLACEMENT};
    if (!pFrm->GetWindowPlacement(&wp)) return;

    CRect *pRc = (CRect *)&wp.rcNormalPosition;
    CSize size = pView->m_sizeClient; // get view size to auto fit

// only when view size is within the mainframe client size
    if (size.cx+sizeNc.cx<=rcMainCli.Width())    
        pRc->right = pRc->left +size.cx +sizeNc.cx;
    if (size.cy+sizeNc.cy<=rcMainCli.Height())    
        pRc->bottom = pRc->top +size.cy +sizeNc.cy;

    if (size.cy+sizeNc.cy> pRc->Height()) 
        pRc->right +=sizeScr.cy;  // need vertical scroll 
    if (size.cx+sizeNc.cx> pRc->Width()) 
        pRc->bottom +=sizeScr.cx; // need horiz scroll 

    if (!pFrm->SetWindowPlacement(&wp))   return;
    pFrm->ShowWindow(wp.showCmd);
}


/////////////////////////////////////////////////////////////////
//  Function: OnWorkspaceCaption()             
//  Note:     message handle, Active window caption toggle
void CMainFrame::OnWorkspaceCaption()
{
    CMDIChildWnd* pFrm = MDIGetActive();  
    if (!pFrm) return;    // no Active child window
    if (pFrm->IsIconic()) return;  // not for minmized child window

    LONG lStyle = GetWindowLong(pFrm->GetSafeHwnd(), GWL_STYLE);  
    BOOL bCap = !!(lStyle & WS_CAPTION);    // casting
    lStyle = bCap ? (lStyle & ~WS_CAPTION) : (lStyle | WS_CAPTION);
    SetWindowLong(pFrm->GetSafeHwnd(), GWL_STYLE, lStyle);       
    
    CRect rc;
    pFrm->GetWindowRect(rc);      
    int iCaption =GetSystemMetrics(SM_CYCAPTION)
                  - GetSystemMetrics(SM_CYBORDER);
    if (bCap) iCaption = -iCaption;

    pFrm->SetWindowPos(NULL, 0, 0, rc.Width(), rc.Height()+iCaption,
                       SWP_NOMOVE|SWP_NOZORDER);
}

void CMainFrame::OnUpdateWWSCaption(CCmdUI* pCmdUI)
{
    CMDIChildWnd* pFrm = MDIGetActive();  
    if (!pFrm) return;    // no Active child window
    pCmdUI->SetCheck(!!(WS_CAPTION &        // int casting
                GetWindowLong(pFrm->GetSafeHwnd(), GWL_STYLE)));
    
}
