// ===========================================================
// gscale.cpp
// WDJ Bitmap Grayscaling Test Application
// Useage: TBTEST <bitmap file name> <mask color>
// Fran Heeran
// ===========================================================
#include <windows.h>
#include "wdjtb.h"

static char s_szTestClass[]  = "WDJ_Test";
static HIMAGELIST s_hImgHot  = NULL;
static HIMAGELIST s_hImgCold = NULL;
static HIMAGELIST s_hImgDisabled = NULL;
static int        s_iImageHeight = 0;
static int        s_iImageWidth = 0;

LRESULT APIENTRY TestWndProc(HWND hWnd, UINT iMsg,
                  WPARAM wParam, LPARAM lParam);

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE  hPrevInst,
                   LPSTR pszCmdLine, int nCmdShow)
{
   MSG         msg;
   COLORREF    clrMask = CLR_NONE;

   // Register the window and create it
   WNDCLASS    wc;
   memset(&wc, 0, sizeof(wc));

   if (!hPrevInst) {
      wc.style          = CS_HREDRAW | CS_VREDRAW;
      wc.lpfnWndProc    = TestWndProc;
      wc.hInstance      = hInst;
      wc.hbrBackground  = (HBRUSH)GetStockObject(LTGRAY_BRUSH);
      wc.lpszClassName  = s_szTestClass;

      if (!RegisterClass(&wc))
         return FALSE;
   }

   // Create main window
   HWND hWnd = CreateWindow(s_szTestClass,
                   "WDJ Toolbar Utility Test", WS_OVERLAPPEDWINDOW,
                   CW_USEDEFAULT, CW_USEDEFAULT,
                   CW_USEDEFAULT, CW_USEDEFAULT, NULL,
                   NULL, hInst, NULL);
   if (!hWnd)
      return 0;

   // Load the image specified
   if (strlen(pszCmdLine)) {
      LPSTR pszClr = strchr(pszCmdLine, ' ');
      if (pszClr) {
         *pszClr++ = 0;
         clrMask = strtol(pszClr, 0, 16);
      }

      WDJLoadToolbarImageLists(NULL, pszCmdLine, -1, &s_hImgHot,
               &s_hImgCold, &s_hImgDisabled, LR_LOADFROMFILE,
               clrMask);
   }

   if (s_hImgHot)
      ImageList_GetIconSize(s_hImgHot, &s_iImageWidth,
                            &s_iImageHeight);

   ShowWindow(hWnd, SW_SHOWNORMAL);

   while (GetMessage(&msg, NULL, 0,0 )) {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }

   return 0;
}

LRESULT APIENTRY TestWndProc(HWND hWnd, UINT iMsg,
                  WPARAM wParam, LPARAM lParam)
{
   switch (iMsg) {
      case WM_DESTROY:
         PostQuitMessage(0);
         break;

      case WM_PAINT: {
         PAINTSTRUCT ps;
         HDC   hDC = BeginPaint(hWnd, &ps);

         if (s_hImgHot)
            ImageList_Draw(s_hImgHot, 0, hDC, 0, 0,
                           ILD_NORMAL);
         if (s_hImgCold)
            ImageList_Draw(s_hImgCold, 0, hDC, 0,
                           s_iImageHeight, ILD_NORMAL);
         if (s_hImgDisabled)
            ImageList_Draw(s_hImgDisabled, 0, hDC, 0,
                           s_iImageHeight * 2, ILD_NORMAL);

         EndPaint(hWnd, &ps);
         break;
      }

      default:
         return (DefWindowProc(hWnd, iMsg, wParam, lParam));
   }
   return 0L;
}

/* end of file */